@@ -537,7 +537,12 @@ def get_data_structure(job, state, args, xml=None, metadata=None):
 
     # add the core count
     if job.corecount and job.corecount != 'null' and job.corecount != 'NULL':
-        data['coreCount'] = mean(job.corecounts) if job.corecounts else job.corecount
+        data['coreCount'] = job.corecount
+        #data['coreCount'] = mean(job.corecounts) if job.corecounts else job.corecount
+    if job.corecounts:
+        _mean = mean(job.corecounts)
+        log.info('mean actualcorecount: %f' % _mean)
+        data['meanCoreCount'] = _mean
 
     # get the number of events, should report in heartbeat in case of preempted.
     if job.nevents != 0:
@@ -1471,6 +1476,9 @@ def retrieve(queues, traces, args):  # noqa: C901
                 jobnumber += 1
                 while not args.graceful_stop.is_set():
                     if has_job_completed(queues, args):
+                        #import signal
+                        #os.kill(os.getpid(), signal.SIGTERM)
+
                         args.job_aborted.clear()
                         args.abort_job.clear()
                         logger.info('ready for new job')
@@ -1910,7 +1918,7 @@ def get_finished_or_failed_job(args, queues):
 def get_heartbeat_period(debug=False):
     """
     Return the proper heartbeat period, as determined by normal or debug mode.
-    In normal mode, the hearbeat period is 30*60 s, while in debug mode it is 5*60 s. Both values are defined in the
+    In normal mode, the heartbeat period is 30*60 s, while in debug mode it is 5*60 s. Both values are defined in the
     config file.
 
     :param debug: Boolean, True for debug mode. False otherwise.
