@@ -12,6 +12,7 @@ from os import environ, getcwd, setpgrp  #, getpgid  #setsid
 from sys import version_info
 
 from pilot.common.errorcodes import ErrorCodes
+from pilot.util.config import config
 
 import logging
 logger = logging.getLogger(__name__)
@@ -121,7 +122,7 @@ def containerise_executable(executable, **kwargs):
         # should a container really be used?
         do_use_container = job.usecontainer if job else container.do_use_container(**kwargs)
         # overrule for event service
-        if job.is_eventservice and do_use_container:
+        if job and job.is_eventservice and do_use_container and config.Payload.executor_type.lower() != 'raythena':
             logger.info('overruling container decision for event service grid job')
             do_use_container = False
 
